%% Clear
clc;
clear;
close all;
format compact;
format short g;

%% Set up fem struct
fem = getInitFem;
prob.fem.dof = getDof(fem);

%% Set up coefficient functions
prob.theta(1).fun = @(mu) mu(1);
prob.theta(2).fun = @(mu) mu(2);

% set parameter value
mu = [1;1]

%% Pre-assemble FEM matrices

% assemble the parameter-independent fem matrices
prob = performPreAssembling(fem,prob);

% Time comparison:
% assemble the full (parameter-dependent) problem from pre-assembled
% matrices
tic
[Kpre,Lpre] = assembleFemPre(prob,mu);
time_assPre = toc

% assemble the full problem directly (for comparison only)
tic;
[K,L] = assemble(fem,'Out',{'K','L'});
time_assAll = toc

max(max(abs(K-Kpre)))
norm(L-Lpre,inf)

%% Solve using comsol
tic
u_comsol = getSol(prob, mu, fem);
time_solveComsol = toc

fem.sol = femsol(u_comsol);
figure;
postplot(fem,'tridata','u');

%% Solve using pre-assembled matrices

tic
u_pre = getSol(prob, mu);
time_solvePre = toc

err = u_comsol - u_pre;
norm(err,'inf')

fem.sol = femsol(err);
figure;
postplot(fem,'tridata','u');

%% Compute coercivity and inf-sup constant

% assemble matrix for discrete H1 inner product
[fem.const{2:2:end}] = deal(1);
[C,D] = assemble(fem,'Out',{'C','D'});
H1 = C+D;
H1 = (H1+H1')/2;

% extract relevant part as inner product matrix
prob.fem.innProd = H1(prob.fem.dof.ind,prob.fem.dof.ind);

% compute coercivities and inf-sup-constants
N = 101;
alpha = zeros(N,1);
beta = zeros(N,1);
mu1_list = logspace(-2,2,N);
mu2 = 1;

for n=1:numel(mu1_list)
  mu = [mu1_list(n);mu2];
  
  alpha(n) = getCoercivity(prob,mu);
  beta(n) = getInfSup(prob,mu);
end

figure(3);
loglog(mu1_list,alpha);
xlabel \mu_1;
ylabel \alpha(\mu_1);
hold
loglog(mu1_list, beta, 'ro');

